Option Compare Binary
Option Explicit On 
Option Strict Off   ' Das Beispiel verwendet sptes Binden.

Imports System
Imports System.Diagnostics
Imports System.IO
Imports System.Reflection

''' <summary>
'''   Stellt den Einsprungspunkt der Anwendung bereit.
''' </summary>
Public Module Program

    ''' <summary>
    '''   Der Einsprungspunkt der Anwendung.
    ''' </summary>
    Public Sub Main()
        For Each FileName As String In Directory.GetFiles(ApplicationPath(), "*.dll")
            If FileVersionInfo.GetVersionInfo(FileName).FileDescription = "PlugIns PlugIn DLL" Then
                Dim PlugIn As Object = _
                    ClassByName( _
                        FileName, _
                        "PlugIn.Main", _
                        New Object() {4, "Hallo"} _
                    )
                PlugIn.SayHello("Frank")
            End If
        Next FileName
    End Sub

    ''' <summary>
    '''   Instanziert eine Klasse aus einem durch den Dateinamen
    '''   definierten Assembly fr die Verwendung mit sptem Binden.
    ''' </summary>
    ''' <param name="AssemblyFileName">
    '''   Der Dateiname des Assembly.
    ''' </param>
    ''' <param name="ClassName">
    '''   Der Name der aus <paramref name="AssemblyFileName"/> zu ladenden
    '''   Klasse.
    ''' </param>
    ''' <param name="Args">
    '''   Die Parameterwerte fr den Konstruktor der zu instanzierenden
    '''   Klasse.
    ''' </param>
    ''' <returns>Die erstellte Instanz der zu ladenden Klasse.</returns>
    ''' <exception cref="System.ApplicationException">
    '''   Tritt ein, wenn beim Laden ein Fehler auftritt.
    ''' </exception>
    Public Function ClassByName( _
        ByVal AssemblyFileName As String, _
        ByVal ClassName As String, _
        ByVal Args() As Object _
    ) As Object
        Dim Result As Object = _
            [Assembly].LoadFrom(AssemblyFileName).CreateInstance( _
                ClassName, _
                False, _
                BindingFlags.CreateInstance, _
                Nothing, _
                Args, _
                Nothing, _
                Nothing _
            )
        If Result Is Nothing Then
            Throw New ApplicationException("The instance could not be created.")
        Else
            Return Result
        End If
    End Function


    ''' <summary>
    '''   Gibt den Pfad der laufenden Anwendung zurck.
    ''' </summary>
    ''' <returns>Der Pfad der laufenden Anwendung.</returns>
    Private Function ApplicationPath() As String
        Return _
            Path.GetDirectoryName([Assembly].GetEntryAssembly().Location)
    End Function
End Module